@@ -1,6 +1,6 @@
 /*
- Copyright (C) 2009-2010 Christian Dywan <christian@twotoasts.de>
- Copyright (C) 2009 Alexander Butenko <a.butenka@gmail.com>
+ Copyright (C) 2009-2012 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2009-2012 Alexander Butenko <a.butenka@gmail.com>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -9,7 +9,6 @@
 
  See the file COPYING for the full license text.
 */
-
 #include <midori/midori.h>
 #include <glib/gstdio.h>
 
@@ -35,13 +34,12 @@
     #define adblock_debug(dmsg, darg1, darg2) /* nothing */
 #endif
 
-static GHashTable* pattern;
-static GHashTable* keys;
-static GHashTable* optslist;
-static GHashTable* urlcache;
-static GString* blockcss;
-static GString* blockcssprivate;
-static gchar* blockscript = NULL;
+static GHashTable* pattern = NULL;
+static GHashTable* keys = NULL;
+static GHashTable* optslist = NULL;
+static GHashTable* urlcache = NULL;
+static GHashTable* blockcssprivate = NULL;
+static GString* blockcss = NULL;
 #ifdef G_ENABLE_DEBUG
 static guint debug;
 #endif
@@ -54,36 +52,83 @@ adblock_reload_rules (MidoriExtension* extension,
                       gboolean         custom_only);
 
 static gchar*
-adblock_build_js (const gchar* private)
+adblock_build_js (const gchar* uri)
 {
-    return g_strdup_printf (
+    gchar* domain;
+    const gchar* style;
+    GString* subdomain;
+    GString* code;
+    int cnt = 0, blockscnt = 0;
+    gchar** subdomains;
+
+    domain = midori_uri_parse_hostname (uri, NULL);
+    subdomains = g_strsplit (domain, ".", -1);
+    g_free (domain);
+    if (!subdomains)
+        return NULL;
+
+    code = g_string_new (
         "window.addEventListener ('DOMContentLoaded',"
         "function () {"
         "   if (document.getElementById('madblock'))"
         "       return;"
-        "   var URL = location.href;"
-        "   var sites = new Array(); %s;"
-        "   var public = '.madblockplaceholder ';"
-        "   for (var i in sites) {"
-        "       if (URL.indexOf(i) != -1 && sites[i] ){"
-        "           public += ', .'+sites[i];"
-        "           break;"
-        "   }}"
-        "   public += ' {display: none !important}';"
+        "   public = '");
+
+    cnt = g_strv_length (subdomains) - 1;
+    subdomain = g_string_new (subdomains [cnt]);
+    g_string_prepend_c (subdomain, '.');
+    cnt--;
+    while (cnt >= 0)
+    {
+        g_string_prepend (subdomain, subdomains[cnt]);
+        if ((style = g_hash_table_lookup (blockcssprivate, subdomain->str)))
+        {
+            g_string_append (code, style);
+            g_string_append_c (code, ',');
+            blockscnt++;
+        }
+        g_string_prepend_c (subdomain, '.');
+        cnt--;
+    }
+    g_string_free (subdomain, TRUE);
+    g_strfreev (subdomains);
+
+    if (blockscnt == 0)
+        return g_string_free (code, TRUE);
+
+    g_string_append (code,
+        "   zz-non-existent {display: none !important}';"
         "   var mystyle = document.createElement('style');"
         "   mystyle.setAttribute('type', 'text/css');"
         "   mystyle.setAttribute('id', 'madblock');"
         "   mystyle.appendChild(document.createTextNode(public));"
         "   var head = document.getElementsByTagName('head')[0];"
         "   if (head) head.appendChild(mystyle);"
-        "}, true);",
-        private);
+        "}, true);");
+    return g_string_free (code, FALSE);
 }
 
 static GString*
 adblock_fixup_regexp (const gchar* prefix,
                       gchar*       src);
 
+static void
+adblock_destroy_db ()
+{
+    if (blockcss)
+        g_string_free (blockcss, TRUE);
+    blockcss = NULL;
+
+    g_hash_table_destroy (pattern);
+    pattern = NULL;
+    g_hash_table_destroy (optslist);
+    optslist = NULL;
+    g_hash_table_destroy (urlcache);
+    urlcache = NULL;
+    g_hash_table_destroy (blockcssprivate);
+    blockcssprivate = NULL;
+}
+
 static void
 adblock_init_db ()
 {
@@ -99,12 +144,13 @@ adblock_init_db ()
     urlcache = g_hash_table_new_full (g_str_hash, g_str_equal,
                    (GDestroyNotify)g_free,
                    (GDestroyNotify)g_free);
+    blockcssprivate = g_hash_table_new_full (g_str_hash, g_str_equal,
+                   (GDestroyNotify)g_free,
+                   (GDestroyNotify)g_free);
+
     if (blockcss && blockcss->len > 0)
         g_string_free (blockcss, TRUE);
-    if (blockcssprivate && blockcssprivate->len > 0)
-        g_string_free (blockcssprivate, TRUE);
     blockcss = g_string_new ("z-non-exist");
-    blockcssprivate = g_string_new ("");
 }
 
 static void
@@ -112,23 +158,9 @@ adblock_download_notify_status_cb (WebKitDownload*  download,
                                    GParamSpec*      pspec,
                                    MidoriExtension* extension)
 {
-    gchar* path;
-    MidoriApp* app;
-    MidoriWebSettings* settings;
-
     if (webkit_download_get_status (download) != WEBKIT_DOWNLOAD_STATUS_FINISHED)
         return;
-
-    path = g_filename_from_uri (webkit_download_get_destination_uri (download), NULL, NULL);
-    adblock_parse_file (path);
-    g_free (path);
-
-    app = midori_extension_get_app (extension);
-    settings = katze_object_get_object (app, "settings");
-    g_string_append (blockcss, " {display: none !important}\n");
-    midori_web_settings_add_style (settings, "adblock-blockcss", blockcss->str);
-    katze_assign (blockscript, adblock_build_js (blockcssprivate->str));
-    g_object_unref (settings);
+    adblock_reload_rules (extension, FALSE);
 }
 
 static gchar*
@@ -167,6 +199,8 @@ adblock_reload_rules (MidoriExtension* extension,
     MidoriApp* app = midori_extension_get_app (extension);
     MidoriWebSettings* settings = katze_object_get_object (app, "settings");
 
+    if (pattern)
+        adblock_destroy_db ();
     adblock_init_db ();
 
     custom_list = g_build_filename (midori_extension_get_config_dir (extension),
@@ -208,7 +242,6 @@ adblock_reload_rules (MidoriExtension* extension,
     g_strfreev (filters);
     g_string_append (blockcss, " {display: none !important}\n");
 
-    katze_assign (blockscript, adblock_build_js (blockcssprivate->str));
     midori_web_settings_add_style (settings, "adblock-blockcss", blockcss->str);
     g_object_unref (settings);
 }
@@ -344,7 +377,8 @@ static void
 adblock_preferences_add_clicked_cb (GtkWidget*    button,
                                     GtkTreeModel* model)
 {
-    GtkEntry* entry = g_object_get_data (G_OBJECT (button), "entry");
+    GtkEntry* entry = GTK_IS_ENTRY (button)
+        ? button : g_object_get_data (G_OBJECT (button), "entry");
     gtk_list_store_insert_with_values (GTK_LIST_STORE (model),
         NULL, 0, 0, gtk_entry_get_text (entry), -1);
     gtk_entry_set_text (entry, "");
@@ -534,6 +568,8 @@ adblock_get_preferences_dialog (MidoriExtension* extension)
     g_object_set_data (G_OBJECT (button), "entry", entry);
     g_signal_connect (button, "clicked",
         G_CALLBACK (adblock_preferences_add_clicked_cb), liststore);
+    g_signal_connect (entry, "activate",
+        G_CALLBACK (adblock_preferences_add_clicked_cb), liststore);
     gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, FALSE, 0);
     button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
     g_object_set_data (G_OBJECT (button), "treeview", treeview);
@@ -593,29 +629,37 @@ adblock_open_preferences_cb (MidoriExtension* extension)
         gtk_window_present (GTK_WINDOW (dialog));
 }
 
-static inline gboolean
-adblock_check_filter_options (GRegex*       regex,
-                              const gchar*  opts,
-                              const gchar*  req_uri,
-                              const gchar*  page_uri)
+static inline gint
+adblock_check_rule (GRegex*      regex,
+                    const gchar* patt,
+                    const gchar* req_uri,
+                    const gchar* page_uri)
 {
-    if (g_regex_match_simple (",third-party", opts,
+    gchar* opts;
+
+    if (!g_regex_match_full (regex, req_uri, -1, 0, 0, NULL, NULL))
+        return FALSE;
+
+    opts = g_hash_table_lookup (optslist, patt);
+    if (opts && g_regex_match_simple (",third-party", opts,
                               G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY))
     {
         if (page_uri && g_regex_match_full (regex, page_uri, -1, 0, 0, NULL, NULL))
-            return TRUE;
+            return FALSE;
     }
     /* TODO: Domain opt check */
-    return FALSE;
+    #ifdef G_ENABLE_DEBUG
+    adblock_debug ("blocked by pattern regexp=%s -- %s", g_regex_get_pattern (regex), req_uri);
+    #endif
+    return TRUE;
 }
 
 static inline gboolean
-adblock_is_matched_by_pattern (const gchar*  req_uri,
-                               const gchar*  page_uri)
+adblock_is_matched_by_pattern (const gchar* req_uri,
+                               const gchar* page_uri)
 {
     GHashTableIter iter;
     gpointer patt, regex;
-    gchar* opts;
 
     if (USE_PATTERN_MATCHING == 0)
         return FALSE;
@@ -623,68 +667,47 @@ adblock_is_matched_by_pattern (const gchar*  req_uri,
     g_hash_table_iter_init (&iter, pattern);
     while (g_hash_table_iter_next (&iter, &patt, &regex))
     {
-        if (g_regex_match_full (regex, req_uri, -1, 0, 0, NULL, NULL))
-        {
-            opts = g_hash_table_lookup (optslist, patt);
-            if (opts && adblock_check_filter_options (regex, opts, req_uri, page_uri) == TRUE)
-                return FALSE;
-            else
-            {
-                adblock_debug ("blocked by pattern regexp=%s -- %s", g_regex_get_pattern (regex), req_uri);
-                return TRUE;
-            }
-        }
+        if (adblock_check_rule (regex, patt, req_uri, page_uri))
+            return TRUE;
     }
     return FALSE;
 }
 
 static inline gboolean
-adblock_is_matched_by_key (const gchar*  req_uri,
-                           const gchar*  page_uri)
+adblock_is_matched_by_key (const gchar* req_uri,
+                           const gchar* page_uri)
 {
     gchar* uri;
     gint len;
     int pos = 0;
     GList* regex_bl = NULL;
     GString* guri;
+    gboolean ret = FALSE;
+    gchar sig[SIGNATURE_SIZE + 1];
 
+    memset (&sig[0], 0, sizeof (sig));
+    /* Signatures are made on pattern, so we need to convert url to a pattern as well */
     guri = adblock_fixup_regexp ("", (gchar*)req_uri);
     uri = guri->str;
     len = guri->len;
 
     for (pos = len - SIGNATURE_SIZE; pos >= 0; pos--)
     {
-        gchar* sig = g_strndup (uri + pos, SIGNATURE_SIZE);
-        GRegex* regex = g_hash_table_lookup (keys, sig);
-        gchar* opts;
+        GRegex* regex;
+        strncpy (sig, uri + pos, SIGNATURE_SIZE);
+        regex = g_hash_table_lookup (keys, sig);
 
-        if (regex && !g_list_find (regex_bl, regex))
-        {
-            if (g_regex_match_full (regex, req_uri, -1, 0, 0, NULL, NULL))
-            {
-                opts = g_hash_table_lookup (optslist, sig);
-                g_free (sig);
-                if (opts && adblock_check_filter_options (regex, opts, req_uri, page_uri))
-                {
-                    g_free (uri);
-                    g_list_free (regex_bl);
-                    return FALSE;
-                }
-                else
-                {
-                    adblock_debug ("blocked by regexp=%s -- %s", g_regex_get_pattern (regex), uri);
-                    g_free (uri);
-                    g_list_free (regex_bl);
-                    return TRUE;
-                }
-            }
-            regex_bl = g_list_prepend (regex_bl, regex);
-        }
-        g_free (sig);
+        /* Dont check if regex is already blacklisted */
+        if (!regex || g_list_find (regex_bl, regex))
+            continue;
+        ret = adblock_check_rule (regex, sig, req_uri, page_uri);
+        if (ret)
+            break;
+        regex_bl = g_list_prepend (regex_bl, regex);
     }
     g_string_free (guri, TRUE);
     g_list_free (regex_bl);
-    return FALSE;
+    return ret;
 }
 
 static gboolean
@@ -774,11 +797,6 @@ adblock_resource_request_starting_cb (WebKitWebView*         web_view,
     if (midori_uri_is_blank (page_uri))
         return;
 
-    /* Never filter the main page itself */
-    if (web_frame == webkit_web_view_get_main_frame (web_view)
-     && webkit_web_frame_get_load_status (web_frame) == WEBKIT_LOAD_PROVISIONAL)
-        return;
-
     req_uri = webkit_network_request_get_uri (request);
     if (!midori_uri_is_http (req_uri)
      || g_str_has_suffix (req_uri, "favicon.ico"))
@@ -937,13 +955,19 @@ adblock_window_object_cleared_cb (WebKitWebView*  web_view,
                                   JSObjectRef     js_window)
 {
     const char *page_uri;
+    gchar* script;
 
-    page_uri = webkit_web_view_get_uri (web_view);
+    page_uri = webkit_web_frame_get_uri (web_frame);
     /* Don't add adblock css into speeddial and about: pages */
-    if (midori_uri_is_blank (page_uri))
+    if (!midori_uri_is_http (page_uri))
         return;
 
-    g_free (sokoke_js_script_eval (js_context, blockscript, NULL));
+    script = adblock_build_js (page_uri);
+    if (!script)
+        return;
+
+    g_free (sokoke_js_script_eval (js_context, script, NULL));
+    g_free (script);
 }
 
 static void
@@ -1094,7 +1118,9 @@ adblock_compile_regexp (GString* gpatt,
             if (!g_regex_match_simple ("[\\*]", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
                 !g_hash_table_lookup (keys, sig))
             {
+                #ifdef G_ENABLE_DEBUG
                 adblock_debug ("sig: %s %s", sig, patt);
+                #endif
                 g_hash_table_insert (keys, sig, regex);
                 g_hash_table_insert (optslist, sig, g_strdup (opts));
                 signature_count++;
@@ -1104,7 +1130,9 @@ adblock_compile_regexp (GString* gpatt,
                 if (g_regex_match_simple ("^\\*", sig, G_REGEX_UNGREEDY, G_REGEX_MATCH_NOTEMPTY) &&
                     !g_hash_table_lookup (pattern, patt))
                 {
+                    #ifdef G_ENABLE_DEBUG
                     adblock_debug ("patt2: %s %s", sig, patt);
+                    #endif
                     g_hash_table_insert (pattern, patt, regex);
                     g_hash_table_insert (optslist, patt, g_strdup (opts));
                 }
@@ -1120,7 +1148,9 @@ adblock_compile_regexp (GString* gpatt,
     }
     else
     {
+        #ifdef G_ENABLE_DEBUG
         adblock_debug ("patt: %s%s", patt, "");
+        #endif
         /* Pattern is a regexp chars */
         g_hash_table_insert (pattern, patt, regex);
         g_hash_table_insert (optslist, patt, g_strdup (opts));
@@ -1141,47 +1171,52 @@ adblock_add_url_pattern (gchar* prefix,
 
     data = g_strsplit (line, "$", -1);
     if (!data || !data[0])
+    {
+        g_strfreev (data);
         return NULL;
+    }
 
     if (data[1] && data[2])
     {
         patt = g_strconcat (data[0], data[1], NULL);
         opts = g_strconcat (type, ",", data[2], NULL);
-        g_strfreev (data);
     }
     else if (data[1])
     {
         patt = data[0];
         opts = g_strconcat (type, ",", data[1], NULL);
-        g_free (data[1]);
     }
     else
     {
         patt = data[0];
-        opts = g_strdup (type);
+        opts = type;
     }
 
     if (g_regex_match_simple ("subdocument", opts,
                               G_REGEX_CASELESS, G_REGEX_MATCH_NOTEMPTY))
     {
-        g_free (patt);
-        g_free (opts);
+        if (data[1] && data[2])
+            g_free (patt);
+        if (data[1])
+            g_free (opts);
+        g_strfreev (data);
         return NULL;
     }
 
     format_patt = adblock_fixup_regexp (prefix, patt);
 
+    #ifdef G_ENABLE_DEBUG
     adblock_debug ("got: %s opts %s", format_patt->str, opts);
+    #endif
     should_free = adblock_compile_regexp (format_patt, opts);
 
-    g_free (opts);
-    g_free (patt);
+    if (data[1] && data[2])
+        g_free (patt);
+    if (data[1])
+        g_free (opts);
+    g_strfreev (data);
 
-    #if G_ENABLE_DEBUG
-    return g_string_free (format_patt, FALSE);
-    #else
     return g_string_free (format_patt, should_free);
-    #endif
 }
 
 static inline void
@@ -1202,6 +1237,22 @@ adblock_frame_add (gchar* line)
     g_string_append (blockcss, line);
 }
 
+static inline void
+adblock_update_css_hash (gchar* domain,
+                         gchar* value)
+{
+    const gchar* olddata;
+    gchar* newdata;
+
+    if ((olddata = g_hash_table_lookup (blockcssprivate, domain)))
+    {
+        newdata = g_strconcat (olddata, " , ", value, NULL);
+        g_hash_table_replace (blockcssprivate, g_strdup (domain), newdata);
+    }
+    else
+        g_hash_table_insert (blockcssprivate, g_strdup (domain), g_strdup (value));
+}
+
 static inline void
 adblock_frame_add_private (const gchar* line,
                            const gchar* sep)
@@ -1227,15 +1278,22 @@ adblock_frame_add_private (const gchar* line,
         domains = g_strsplit (data[0], ",", -1);
         for (i = 0; domains[i]; i++)
         {
-            g_string_append_printf (blockcssprivate, ";sites['%s']+=',%s'",
-                g_strstrip (domains[i]), data[1]);
+            gchar* domain;
+
+            domain = domains[i];
+            /* Ignore Firefox-specific option */
+            if (!g_strcmp0 (domain, "~pregecko2"))
+                continue;
+            /* strip ~ from domain */
+            if (domain[0] == '~')
+                domain++;
+            adblock_update_css_hash (g_strstrip (domain), data[1]);
         }
         g_strfreev (domains);
     }
     else
     {
-        g_string_append_printf (blockcssprivate, ";sites['%s']+=',%s'",
-            data[0], data[1]);
+        adblock_update_css_hash (data[0], data[1]);
     }
     g_strfreev (data);
 }
@@ -1243,12 +1301,10 @@ adblock_frame_add_private (const gchar* line,
 static gchar*
 adblock_parse_line (gchar* line)
 {
-    if (!line)
-        return NULL;
-    g_strchomp (line);
-    /* Ignore comments and new lines */
-    if (line[0] == '!')
+    /* Skip invalid, empty and comment lines */
+    if (!(line && line[0] != ' ' && line[0] != '!' && line[0]))
         return NULL;
+
     /* FIXME: No support for whitelisting */
     if (line[0] == '@' && line[1] == '@')
         return NULL;
@@ -1256,9 +1312,7 @@ adblock_parse_line (gchar* line)
     if (line[0] == '[')
         return NULL;
 
-    /* Skip garbage */
-    if (line[0] == ' ' || !line[0])
-        return NULL;
+    g_strchomp (line);
 
     /* Got CSS block hider */
     if (line[0] == '#' && line[1] == '#' )
@@ -1276,13 +1330,13 @@ adblock_parse_line (gchar* line)
         adblock_frame_add_private (line, "##");
         return NULL;
     }
-
     /* Got per domain CSS hider rule. Workaround */
     if (strchr (line, '#'))
     {
         adblock_frame_add_private (line, "#");
         return NULL;
     }
+
     /* Got URL blocker rule */
     if (line[0] == '|' && line[1] == '|' )
     {
@@ -1351,16 +1405,8 @@ adblock_deactivate_cb (MidoriExtension* extension,
         browser, adblock_add_tab_cb, extension);
     midori_browser_foreach (browser, (GtkCallback)adblock_deactivate_tabs, browser);
 
-    if (blockcss)
-        g_string_free (blockcss, TRUE);
-    if (blockcssprivate)
-        g_string_free (blockcssprivate, TRUE);
-
+    adblock_destroy_db ();
     midori_web_settings_remove_style (settings, "adblock-blockcss");
-    blockcssprivate = blockcss = NULL;
-    g_hash_table_destroy (pattern);
-    g_hash_table_destroy (optslist);
-    g_hash_table_destroy (urlcache);
     g_object_unref (settings);
 }
 
@@ -1491,7 +1537,7 @@ extension_init (void)
     MidoriExtension* extension = g_object_new (MIDORI_TYPE_EXTENSION,
         "name", _("Advertisement blocker"),
         "description", _("Block advertisements according to a filter list"),
-        "version", "0.5" MIDORI_VERSION_SUFFIX,
+        "version", "0.6" MIDORI_VERSION_SUFFIX,
         "authors", "Christian Dywan <christian@twotoasts.de>",
         NULL);
     midori_extension_install_string_list (extension, "filters", NULL, G_MAXSIZE);
